@@ -24,7 +24,8 @@
 #
 ###############################################################################
 
-__version__ = u'0.13.0'
-"""
-AutobahnPython library version.
-"""
+from __future__ import absolute_import
+
+from autobahn._version import __version__
+
+version = __version__
