@@ -80,19 +80,17 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
     Adapter class for Twisted WebSocket client and server protocols.
     """
 
-    peer = '<never connected>'
+    peer = u'<never connected>'
 
     log = txaio.make_logger()
 
     def connectionMade(self):
         # the peer we are connected to
         try:
-            peer = self.transport.getPeer()
+            self.peer = peer2str(self.transport.getPeer())
         except AttributeError:
             # ProcessProtocols lack getPeer()
-            self.peer = "process {}".format(self.transport.pid)
-        else:
-            self.peer = peer2str(peer)
+            self.peer = u'process:{}'.format(self.transport.pid)
 
         self._connectionMade()
         self.log.debug('Connection made to {peer}', peer=self.peer)
@@ -302,20 +300,19 @@ class WrappingWebSocketAdapter(object):
     """
 
     def onConnect(self, requestOrResponse):
-
         # Negotiate either the 'binary' or the 'base64' WebSocket subprotocol
         if isinstance(requestOrResponse, ConnectionRequest):
             request = requestOrResponse
             for p in request.protocols:
                 if p in self.factory._subprotocols:
-                    self._binaryMode = (p != 'base64')
+                    self._binaryMode = (p != u'base64')
                     return p
             raise ConnectionDeny(ConnectionDeny.NOT_ACCEPTABLE, u'this server only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
         elif isinstance(requestOrResponse, ConnectionResponse):
             response = requestOrResponse
             if response.protocol not in self.factory._subprotocols:
-                self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, 'this client only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
-            self._binaryMode = (response.protocol != 'base64')
+                self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, u'this client only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
+            self._binaryMode = (response.protocol != u'base64')
         else:
             # should not arrive here
             raise Exception("logic error")
@@ -325,13 +322,13 @@ class WrappingWebSocketAdapter(object):
 
     def onMessage(self, payload, isBinary):
         if isBinary != self._binaryMode:
-            self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, 'message payload type does not match the negotiated subprotocol')
+            self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, u'message payload type does not match the negotiated subprotocol')
         else:
             if not isBinary:
                 try:
                     payload = b64decode(payload)
                 except Exception as e:
-                    self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, 'message payload base64 decoding error: {0}'.format(e))
+                    self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, u'message payload base64 decoding error: {0}'.format(e))
             self._proto.dataReceived(payload)
 
     # noinspection PyUnusedLocal
@@ -388,8 +385,7 @@ class WrappingWebSocketServerFactory(WebSocketServerFactory):
                  reactor=None,
                  enableCompression=True,
                  autoFragmentSize=0,
-                 subprotocol=None,
-                 debug=False):
+                 subprotocol=None):
         """
 
         :param factory: Stream-based factory to be wrapped.
@@ -398,15 +394,14 @@ class WrappingWebSocketServerFactory(WebSocketServerFactory):
         :type url: unicode
         """
         self._factory = factory
-        self._subprotocols = ['binary', 'base64']
+        self._subprotocols = [u'binary', u'base64']
         if subprotocol:
             self._subprotocols.append(subprotocol)
 
         WebSocketServerFactory.__init__(self,
                                         url=url,
                                         reactor=reactor,
-                                        protocols=self._subprotocols,
-                                        debug=debug)
+                                        protocols=self._subprotocols)
 
         # automatically fragment outgoing traffic into WebSocket frames
         # of this size
@@ -453,8 +448,7 @@ class WrappingWebSocketClientFactory(WebSocketClientFactory):
                  reactor=None,
                  enableCompression=True,
                  autoFragmentSize=0,
-                 subprotocol=None,
-                 debug=False):
+                 subprotocol=None):
         """
 
         :param factory: Stream-based factory to be wrapped.
@@ -463,15 +457,14 @@ class WrappingWebSocketClientFactory(WebSocketClientFactory):
         :type url: unicode
         """
         self._factory = factory
-        self._subprotocols = ['binary', 'base64']
+        self._subprotocols = [u'binary', u'base64']
         if subprotocol:
             self._subprotocols.append(subprotocol)
 
         WebSocketClientFactory.__init__(self,
                                         url=url,
                                         reactor=reactor,
-                                        protocols=self._subprotocols,
-                                        debug=debug)
+                                        protocols=self._subprotocols)
 
         # automatically fragment outgoing traffic into WebSocket frames
         # of this size
@@ -533,7 +526,7 @@ def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
 
     if factory.proxy is not None:
         factory.contextFactory = contextFactory
-        conn = reactor.connectTCP(factory.proxy['host'], factory.proxy['port'], factory, timeout, bindAddress)
+        conn = reactor.connectTCP(factory.proxy[u'host'], factory.proxy[u'port'], factory, timeout, bindAddress)
     else:
         if factory.isSecure:
             conn = reactor.connectSSL(factory.host, factory.port, factory, contextFactory, timeout, bindAddress)
