@@ -243,11 +243,12 @@ class JsonSerializer(Serializer):
 ISerializer.register(JsonSerializer)
 
 
-#
-# MsgPack serialization depends on the `msgpack` package being available
+# MsgPack serialization depends on the `u-msgpack` package being available
+# https://pypi.python.org/pypi/u-msgpack-python
+# https://github.com/vsergeev/u-msgpack-python
 #
 try:
-    import msgpack
+    import umsgpack
 except ImportError:
     pass
 else:
@@ -259,15 +260,8 @@ else:
         Flag that indicates whether this serializer needs a binary clean transport.
         """
 
-        ENABLE_V5 = True
-        """
-        Enable version 5 of the MsgPack specification (which differentiates
-        between strings and binary).
-        """
-
         def __init__(self, batched=False):
             """
-            Ctor.
 
             :param batched: Flag that controls whether serializer operates in batched mode.
             :type batched: bool
@@ -278,7 +272,7 @@ else:
             """
             Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
             """
-            data = msgpack.packb(obj, use_bin_type=self.ENABLE_V5)
+            data = umsgpack.packb(obj)
             if self._batched:
                 return struct.pack("!L", len(data)) + data
             else:
@@ -289,22 +283,6 @@ else:
             Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
             """
 
-            def ensure_string_keys(d):
-                """
-                under python 2, with use_bin_type=True, most dict keys end up
-                getting encoded as bytes (any syntax except {u"key":
-                u"value"}) so instead of recursively looking through
-                everything that's getting serialized, we fix them up
-                on the way out using msgpack's `object_hook` as
-                there's no corresponding hook for serialization...
-                """
-                for (k, v) in six.iteritems(d):
-                    if not isinstance(k, six.text_type):
-                        newk = six.text_type(k, encoding='utf8')
-                        del d[k]
-                        d[newk] = v
-                return d
-
             if self._batched:
                 msgs = []
                 N = len(payload)
@@ -321,13 +299,7 @@ else:
                     data = payload[i + 4:i + 4 + l]
 
                     # append parsed raw message
-                    msgs.append(
-                        msgpack.unpackb(
-                            data,
-                            encoding='utf-8',
-                            object_hook=ensure_string_keys,
-                        )
-                    )
+                    msgs.append(umsgpack.unpackb(data))
 
                     # advance until everything consumed
                     i = i + 4 + l
@@ -337,11 +309,7 @@ else:
                 return msgs
 
             else:
-                unpacked = msgpack.unpackb(
-                    payload,
-                    encoding='utf-8',
-                    object_hook=ensure_string_keys,
-                )
+                unpacked = umsgpack.unpackb(payload)
                 return [unpacked]
 
     IObjectSerializer.register(MsgPackObjectSerializer)
@@ -492,3 +460,111 @@ else:
     ISerializer.register(CBORSerializer)
 
     __all__.append('CBORSerializer')
+
+
+# UBJSON serialization depends on the `py-ubjson` package being available
+# https://pypi.python.org/pypi/py-ubjson
+# https://github.com/Iotic-Labs/py-ubjson
+try:
+    import ubjson
+except ImportError:
+    pass
+else:
+
+    class UBJSONObjectSerializer(object):
+
+        BINARY = True
+        """
+        Flag that indicates whether this serializer needs a binary clean transport.
+        """
+
+        def __init__(self, batched=False):
+            """
+
+            :param batched: Flag that controls whether serializer operates in batched mode.
+            :type batched: bool
+            """
+            self._batched = batched
+
+        def serialize(self, obj):
+            """
+            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.serialize`
+            """
+            data = ubjson.dumpb(obj)
+            if self._batched:
+                return struct.pack("!L", len(data)) + data
+            else:
+                return data
+
+        def unserialize(self, payload):
+            """
+            Implements :func:`autobahn.wamp.interfaces.IObjectSerializer.unserialize`
+            """
+
+            if self._batched:
+                msgs = []
+                N = len(payload)
+                i = 0
+                while i < N:
+                    # read message length prefix
+                    if i + 4 > N:
+                        raise Exception("batch format error [1]")
+                    l = struct.unpack("!L", payload[i:i + 4])[0]
+
+                    # read message data
+                    if i + 4 + l > N:
+                        raise Exception("batch format error [2]")
+                    data = payload[i + 4:i + 4 + l]
+
+                    # append parsed raw message
+                    msgs.append(ubjson.loadb(data))
+
+                    # advance until everything consumed
+                    i = i + 4 + l
+
+                if i != N:
+                    raise Exception("batch format error [3]")
+                return msgs
+
+            else:
+                unpacked = ubjson.loadb(payload)
+                return [unpacked]
+
+    IObjectSerializer.register(UBJSONObjectSerializer)
+
+    __all__.append('UBJSONObjectSerializer')
+
+    class UBJSONSerializer(Serializer):
+
+        SERIALIZER_ID = u"ubjson"
+        """
+        ID used as part of the WebSocket subprotocol name to identify the
+        serializer with WAMP-over-WebSocket.
+        """
+
+        RAWSOCKET_SERIALIZER_ID = 4
+        """
+        ID used in lower four bits of second octet in RawSocket opening
+        handshake identify the serializer with WAMP-over-RawSocket.
+        """
+
+        MIME_TYPE = u"application/ubjson"
+        """
+        MIME type announced in HTTP request/response headers when running
+        WAMP-over-Longpoll HTTP fallback.
+        """
+
+        def __init__(self, batched=False):
+            """
+            Ctor.
+
+            :param batched: Flag to control whether to put this serialized into batched mode.
+            :type batched: bool
+            """
+            Serializer.__init__(self, UBJSONObjectSerializer(batched=batched))
+            if batched:
+                self.SERIALIZER_ID = u"ubjson.batched"
+
+    ISerializer.register(UBJSONSerializer)
+
+    __all__.append('UBJSONSerializer')
